#include <string.h>

#include "..\Shared\Taunts.h"
#include "InstantMission.h"

//#include <windows.h>
/*
InstantMission

Just add water!!
*/

// Distance from enemy recycler for enemies to be parked for siege
// mode to be triggered.
const float kfSIEGE_DISTANCE = 250.0f;

const int kRACE_ISDF = 105; // ascii for 'i'
const int kRACE_SCION = 102; // ascii for 'f'

// Base AIP name for stock AIPs. 
const char *kStockAIPNameBase="stock13_";

//#define TEST_INSTANT 1

//
// Errant scav-cleaning code. Added by Nathan Mates 7/20/01.
//
// By the way, this is mostly based on the cleanup of extra ships in
// the DM DLL. 
//

const char *AIPTypeExtensions="0123als"; // AIPType enum is offset in this string table

// Constructor
instantMission::instantMission(void)
{
	b_count = &b_last - &b_first - 1;
	b_array = &b_first + 1;

	f_count = &f_last - &f_first - 1;
	f_array = &f_first + 1;

	h_count = &h_last - &h_first - 1;
	h_array = &h_first + 1;

	i_count = &i_last - &i_first - 1;
	i_array = &i_first + 1;

	mCustomAIPStr = (char*)(&mCustomAIPInts[0]);
	mCustomAIPStrLen = sizeof(mCustomAIPInts);
}

// Builds a starting vehicle on the specified team with either ODF1
// (preferred, if found) or ODF2 as its name, modified by the
// race. Sets group, if specified. Returns the handle
Handle instantMission::BuildStartingVehicle(int aTeam, int aRace, char *ODF1, char *ODF2, char *Where, int aGrp)
{
	char TempODF[32];
	memset(TempODF,0,sizeof(TempODF));

	strncpy(TempODF,ODF1,sizeof(TempODF)-1);
	TempODF[0] = (char)aRace;
	if(!DoesODFExist(TempODF)) { // doesn't exist, so fallback to ODF2
		memset(TempODF,0,sizeof(TempODF));
		strncpy(TempODF,ODF2,sizeof(TempODF)-1);
		TempODF[0] = (char)aRace;
	}

	Handle h = BuildObject(TempODF,aTeam,Where);
	if((h != 0) && (aGrp >= 0))
		SetGroup(h,aGrp);
	return h;
}


// Sets the current AIP based on CPU's race, and AIP type.
void instantMission::SetCPUAIPlan(AIPType Type)
{
	if((Type<0) || (Type >=MAX_AIP_TYPE))
		Type=AIPType3; // Default

	// Try to keep AI from getting hung up if user rushes them - NM 7/15/05
	if((Type > AIPType3) && (!bCPUBuiltCommBunker))
		return;

	char AIPFile[512];
	unsigned long l; // End of string

	// Prezap contents of string, so that we can safely append
	// characters w/o worrying about having to re-terminate
	memset(AIPFile,0,sizeof(AIPFile));

	// If custom AIPs were specified, use them.
	if(mCustomAIPStr[0]) {
		mCustomAIPStr[mCustomAIPStrLen - 1] = 0; // re-terminate strings

		// -16 == enough space to fit chars we add
		strncpy(AIPFile,mCustomAIPStr,sizeof(AIPFile) - 16);
	}
	else {
		// -16 == enough space to fit chars we add
		strncpy(AIPFile,kStockAIPNameBase,sizeof(AIPFile) - 16);
	}

	l=strlen(AIPFile);
	// First extension char: cpu's race
	AIPFile[l++]=(char)CPUTeamRace;

	// Second extension char: human's race.
	AIPFile[l++]=(char)HumanTeamRace;

	AIPFile[l++]=AIPTypeExtensions[Type];

	// Always tack on file extension
	strcat(AIPFile,".aip");
	SetPlan(AIPFile,comp_team);

	if(SetFirstAIP)
		DoTaunt(TAUNTS_Random);
	SetFirstAIP=true;
}

// And set up extra race-specific vehicles at pathpoints in the map.
void instantMission::SetupExtraVehicles(void)
{
	int pathCount;
	char **pathNames;
	GetAiPaths(pathCount, pathNames);

	char cCPUTeamRace = (char) CPUTeamRace;
	char cHumanTeamRace = (char) HumanTeamRace;
	size_t len;

	NumCPUScavs=0; // Nothing in this list now.

	for (int i = 0; i < pathCount; ++i) {
		char *Label = pathNames[i];
		if(strncmp(Label,"mpi",3)==0) {
			// Starts with MPI. Process it.

			const int MaxODFLen=32;
			char ODF1[MaxODFLen];
			char ODF2[MaxODFLen];

			// Prezap full string contents so we don't have to null-term later
			memset(ODF1,0,sizeof(ODF1));
			memset(ODF2,0,sizeof(ODF2));

			char *pUnderscore = strchr(Label,'_');
			if(pUnderscore == NULL)
				continue; // Misformat! No _ found! Bail!

			char *pUnderscore2 = strchr(pUnderscore+1,'_');
			if(pUnderscore2 == NULL) {
				// Only 1 underscore in the name, e.g. (mpic#_ivturr). Copy
				// everything after it to the first name.
				strncpy(ODF1,pUnderscore+1,sizeof(ODF1) - 1);
			}
			else {
				// 2 underscores in the name, e.g. (mpic#_ivturr_fvturr). Copy
				// everything between them to ODF1, rest to ODF2.
				size_t len = (pUnderscore2 - pUnderscore) - 1;
				if(len > (sizeof(ODF2) - 1))
					len = (sizeof(ODF2) - 1);
				strncpy(ODF1,pUnderscore+1,len);
				strncpy(ODF2,pUnderscore2+1,sizeof(ODF2) - 1);
			}

			len = strlen(ODF1);
			if((len > 0) && (ODF1[len-1] == '_'))
				ODF1[len-1] = '\0';
			len = strlen(ODF2);
			if((len > 0) && (ODF2[len-1] == '_'))
				ODF2[len-1] = '\0';

			// For the CPU?
			if(strncmp(Label,"mpic",4)==0) {
				if(ODF1[0] == cCPUTeamRace)
					BuildObject(ODF1,comp_team,Label);
				if(ODF2[0] == cCPUTeamRace)
					BuildObject(ODF2,comp_team,Label);
			}
			else if(strncmp(Label,"mpiC",4)==0) {
				if(ODF1[0]) {
					ODF1[0]= cCPUTeamRace;
					BuildObject(ODF1,comp_team,Label);
				}
				if(ODF2[0]) {
					ODF2[0] = cCPUTeamRace;
					BuildObject(ODF2,comp_team,Label);
				}
			}
			else if(strncmp(Label,"mpih",4)==0) { // For the human side
				Handle h;
				if(ODF1[0] == cHumanTeamRace) {
					h=BuildObject(ODF1,strat_team,Label);
					SetGroup(h,4);
				}

				if(ODF2[0] == cHumanTeamRace) {
					h=BuildObject(ODF2,strat_team,Label);
					SetGroup(h,4);
				}
			}
			else if(strncmp(Label,"mpiH",4)==0) { // For the human side
				Handle h;
				if(ODF1[0]) {
					ODF1[0] = cHumanTeamRace;
					h=BuildObject(ODF1,strat_team,Label);
					SetGroup(h,4);
				}

				if(ODF2[0]) {
					ODF2[0] = cHumanTeamRace;
					h=BuildObject(ODF2,strat_team,Label);
					SetGroup(h,4);
				}
			}
		}
	}
}


void instantMission::Setup(void)
{
	/*
	Here's where you set the values at the start.  
	*/
	start_done=false;
	time_count=1;
	inspect_bunker=false;
	second_objective=false;
	post_bunker=1;
	enemy_reinforcement_time=2000;
	friend_reinforcement_time=2500;
	end_counter=0;
	comp_team=5;
	strat_team=0;  // the default
	dll_flag=GetInstantFlag();
	obj1=(-1);

	turn_counter=0;
	NumCPUScavs=0; // Nothing in this list now.


	siege_on=false;
	anti_assault=false;
	siege_counter=0;
	assault_counter=false;
	late_game=false;
	have_armory=false;

	powerup1=NULL;
	powerup2=NULL;
	powerup3=NULL;
	powerup4=NULL;

	PreloadODF("ivrecy");
	PreloadODF("fvrecy");
	PreloadODF("ivrecycpu");
	PreloadODF("fvrecycpu");

	LastTauntPrintedAt=-2000; // Force first message to be printed...
	DoTaunt(TAUNTS_GameStart);
}

// Note: *NEVER* remove this object from within the AddObject() call.
// It can cause crashes. If you must delete it, wait until the next
// Execute() or something.  NM 3/22/02
void instantMission::AddObject(Handle h)
{
	// Always grab the ODF name out, if possible. Makes life much easier
	// in here.
	char ODFName[64];
	if(GetObjInfo(h, Get_CFG, ODFName)) {
		// Local varb is now setup... 
	}
	else {
		// Failed to get anything. Uhoh. Clear this so our strcmps will be
		// guaranteed to fail.
		memset(ODFName,0,sizeof(ODFName));
	}

	char ObjClass[64];
	GetObjInfo(h, Get_GOClass, ObjClass);

	// Always get one random # in 0..1, for scion shield randomization
	float fRandomNum = GetRandomFloat(1.0f);

	int Team = GetTeamNum(h);

	bool IsCommBunker = (stricmp(ObjClass, "CLASS_COMMBUNKER") == 0) || 
		(stricmp(ObjClass, "CLASS_COMMTOWER") == 0);
	if(IsCommBunker && (Team == comp_team))
		bCPUBuiltCommBunker = true;

	bool IsRecyVehicle = (stricmp(ObjClass, "CLASS_RECYCLERVEHICLE") == 0);
	if(IsRecyVehicle) {
		// If we're not tracking a recycler vehicle for this team right
		// now, store it.
		if(Team == comp_team) {
			if(enemy_recycler == 0)
				enemy_recycler = h;
		}
		if(Team == strat_team) {
			recycler = h;
		}
	}

	if (Team == comp_team)
	{
		// Scav cleanup code added NM 7/20/01.

		// Better way to identify scavs added NM 1/29/05 -- checks for
		// regular & hoverscavs, no matter what their ODF is named.
		char ObjClass[64];
		GetObjInfo(h, Get_GOClass, ObjClass);

		if((stricmp(ObjClass, "CLASS_SCAVENGER") == 0) || (stricmp(ObjClass, "CLASS_SCAVENGERH") == 0)) {
			// Ok, adding in another scav. Add to list (and don't trash
			// memory) This is a fast operation in all cases.
			if(NumCPUScavs<MaxCPUScavs)
				CPUScavList[NumCPUScavs++] = h;

			// Cleanup time? This cleanup is only triggered when the list
			// gets too big, so there's no drag on performance. 
			if(NumCPUScavs>=MaxCPUScavs) {
				int i,j;

				DoTaunt(TAUNTS_Random);

#ifdef _DEBUG
				// Debugging info to let people know this has happened.
				AddToMessagesBox("Cleaning up extraneous scavs! Need to tweak AIPs?\n");
#endif

				for(i=0,j=0;i<MaxCPUScavs;i++) {
					// Because we don't track the cases where scavs could
					// disappear from our list (destroyed, sniped, deployed), we
					// need to verify that it's still a vehicle scav on the same
					// team as it started on before we even consider destroying
					// it. If it doesn't meet those conditions, we no longer
					// need to keep track of it, as it's not a concern anymore.
					//
					// The oldest MinCPUScavs get to live. (CPUScavList[0] is
					// the oldest scav). Theoretically, the oldest scavs are the
					// furthest from the recycler, so killing them shouldn't
					// affect the gameplay too much. You could probably improve
					// this algorithm by keeping, say 10-15 scavs, that are more
					// than 150 meters from the recycler. That is left as an
					// exercise for the reader. :)

					if(IsAround(CPUScavList[i]) && (GetTeamNum(CPUScavList[i]) == comp_team)) {

						// Still should be in the list. Keep it, or destroy?
						if((j<MinCPUScavs) || (h == CPUScavList[i]))
							CPUScavList[j++]=CPUScavList[i];
						else {
							// Don't generate a scrapfield the AI might try and
							// build more scavs to pick up and loop infinitely.
							SetNoScrapFlagByHandle(CPUScavList[i]);
							RemoveObject(CPUScavList[i]);
						}
					}
				} // Loop over all scavs in the CPUScavList[] array.

				// Note how many scavs there are now. This may be smaller than
				// MinCPUScavs if it's decided they don't need to be tracked
				// anymore (see criteria above)
				NumCPUScavs=j;

				// Clean up rest of list. [A bunch of 0s compress better when
				// transmitting across the network. Not that this is going to
				// kill the netcode to not do this, but I like doing this
				// anyhow.]
				for(i=NumCPUScavs;i<MaxCPUScavs;i++)
					CPUScavList[i]=0;

			} // Cleanup time triggered.
		} // ivscav or fvscav has was just created
		// end scav cleanup code. 


		SetSkill(h,difficulty+1);
		if ((IsOdf(h,"ibarmo")) || (IsOdf(h,"fbstro"))) {
			have_armory=true;
		}
		if (have_armory) {
			if(strnicmp(ODFName,"ivtank",6) == 0) {
				GiveWeapon(h,"gspstab_c");
			}
			if(strnicmp(ODFName,"fvtank",6) == 0) {
				GiveWeapon(h,"garc_c");   // what a crutch!  
				//				GiveWeapon(h,"gshield");
			}

			// Randomize shields somewhat. - NM 5/24/03
			if(strnicmp(ODFName,"fv",2) == 0) {
				if(fRandomNum < 0.3f)
					GiveWeapon(h,"gshield");
				else if(fRandomNum < 0.6f)
					GiveWeapon(h,"gabsorb");
				else if(fRandomNum < 0.9f)
					GiveWeapon(h,"gdeflect");
			}

		}
#ifdef _DEBUG
		// Nerf for testing.
		SetSkill(h,0);
		SetMaxHealth(h,50);
		SetCurHealth(h,50);
		SetMaxAmmo(h,50);
		SetCurAmmo(h,50);
#endif
	} // on computer team
	else
		if ((myGoal==0) && (Team == strat_team))
		{

			// if it's artillery bomber/ go to late game tactics
			if((strnicmp(ODFName,"fvartl",6) == 0) ||
				(strnicmp(ODFName,"ivbomb",6) == 0)) {
					if ((!siege_on) && (!late_game)) {
						late_game=true;
						SetCPUAIPlan(AIPTypeL);
					}
				}

				if (( strnicmp(ODFName,"ivscout",7) == 0) 
					|| (strnicmp(ODFName,"ivmisl" ,6) == 0)
					|| (strnicmp(ODFName,"ivtank" ,6) == 0)
					|| (strnicmp(ODFName,"ivrckt" ,6) == 0)
					|| (strnicmp(ODFName,"ivatank",7) == 0)
					|| (strnicmp(ODFName,"ivwalk" ,6) == 0)
					|| (strnicmp(ODFName,"ivserv" ,6) == 0)
					|| (strnicmp(ODFName,"fvscout",7) == 0) 
					|| (strnicmp(ODFName,"fvsent" ,6) == 0)
					|| (strnicmp(ODFName,"fvtank" ,6) == 0)
					|| (strnicmp(ODFName,"fvarch" ,6) == 0)
					|| (strnicmp(ODFName,"fvatank",7) == 0)
					|| (strnicmp(ODFName,"fvwalk" ,6) == 0)
					|| (strnicmp(ODFName,"fvserv" ,6) == 0)
					)
				{
					SetSkill(h,3-difficulty);
					SetTeamNum(h,1);
					//			int grp=GetFirstEmptyGroup();			
					//			SetGroup(h,grp);
				}
		}

		if (Team == strat_team) {

			if(strnicmp(ODFName+1,"brecy",5) == 0) {
				if(!bPastAIP0) {
					bPastAIP0 = true;
					/*
					SetAip here based on turn_counter
					*/

					int stratchoice = turn_counter%2;
					if (CPUTeamRace == kRACE_SCION) { // CPU is scion
						stratchoice = turn_counter%2;
						switch (stratchoice) {
					default:
					case 0:
						SetCPUAIPlan(AIPType1);
						break;
					case 1:
						SetCPUAIPlan(AIPType3);
						break;
					case 2:
						SetCPUAIPlan(AIPType2);
						break;
						}
					}
					else { // CPU is human
						switch (stratchoice % 2) {
					default:
					case 0:
						SetCPUAIPlan(AIPType1);
						break;
					case 1:
						SetCPUAIPlan(AIPType3);
						break;
						}

					}
				} // not past AIP 0
			} else {
				if((strnicmp(ODFName+1,"vatank",6) == 0) ||
					(strnicmp(ODFName+1,"vwalk",5) == 0))
					assault_counter++;
			} // not a recy
		}

		if((!bPastAIP0) && (turn_counter > (180 * 10))) {
			bPastAIP0 = true;
			int stratchoice = turn_counter%2;
			switch (stratchoice % 2) {
		default:
		case 0:
			SetCPUAIPlan(AIPType1);
			break;
		case 1:
			SetCPUAIPlan(AIPType3);
			break;
			}
		}
}

void instantMission::DeleteObject(Handle h)
{
	if (GetTeamNum(h)==strat_team) {
		char ODFName[64];
		GetObjInfo(h, Get_CFG, ODFName);

		// unit exists. Check skipping race name, and any suffix
		if((strnicmp(ODFName+1,"vatank",6) == 0) ||
			(strnicmp(ODFName+1,"vwalk",5) == 0))
			assault_counter--;
	}
	else {
		// CPU team.
		if ((IsOdf(h,"ibarmo")) || (IsOdf(h,"fbstro"))) {
			have_armory=false;
		}
	}

}

void instantMission::CreateObjectives()
{
	goalval=GetInstantGoal();
	switch (goalval)
	{
	case 0:  //ctf
		obj1=BuildObject("apcamr",0,"flag");
		if (obj1!=NULL)
		{
			SetObjectiveOn(obj1);
			SetObjectiveName(obj1,"Capture Target");
			ClearObjectives();
			AddObjective("inst0101.otf",WHITE,8.0f);
		}
		break;
	case 1: // attack
		obj1=BuildObject("mbruin01",comp_team,"objective1");
		if (obj1!=NULL)
		{
			SetObjectiveOn(obj1);
			SetObjectiveName(obj1,"Destroy Target");  // later this could be health/ ammo
			ClearObjectives();
			AddObjective("inst0102.otf",WHITE,8.0f);
			// we need to know if the goal is 
			// protect-destory
		}
		else (obj1=(-1));  // it never existed

		break;
	case 2: // defend
		obj1=BuildObject("mbruin01",1,"objective2");
		if (obj1!=NULL)
		{
			SetObjectiveOn(obj1);
			SetObjectiveName(obj1,"Protect Target");
			ClearObjectives();
			AddObjective("inst0103.otf",WHITE,8.0f);
		}
		break;
	}
	/*
	Again, if this is a strategy instant action..
	some variables should be passed
	*/
}

void instantMission::TestObjectives()
{
	/*
	switch (goalval)
	{

	case 0:  //ctf
	if ((obj1!=(-1)) && (GetDistance(obj1,recycler)<100.0f))
	{
	if (end_counter==0)
	{
	ClearObjectives();
	AddObjective("inst0104.otf",GREEN,8.0f);
	SucceedMission(5.0f,"inst0104.otf");
	}
	end_counter++;
	}
	break;
	case 1:
	// this should vary depending on objective
	if ((obj1!=(-1))  && (!IsAlive(obj1)))
	{
	if (end_counter==0)
	{
	ClearObjectives();
	AddObjective("inst0105.otf",GREEN,8.0f);
	}
	end_counter++;
	if (end_counter==100)
	{
	SucceedMission(1.0f,"inst0105.otf");
	}
	}
	break;
	case 2:
	break;
	}
	*/
	if ((end_counter==0) && (!IsAlive(enemy_recycler)))
	{
		DoTaunt(TAUNTS_CPURecyDestroyed);
		SucceedMission(GetTime()+5.0f,"instantw.txt");
		end_counter++;
	}

	if ((end_counter==0) && (!IsAlive(recycler)))
	{
		DoTaunt(TAUNTS_HumanRecyDestroyed);
		FailMission(GetTime()+5.0f,"instantl.txt");
		end_counter++;
	}

}
void instantMission::Execute(void)
{
	ElapsedGameTime++;
	player=GetPlayerHandle();
	turn_counter++;
	DoGenericStrategy();
}

bool instantMission::Load(bool missionSave)
{
	// Always do this to hook up clients with the taunt engine as well.
	InitTaunts(&ElapsedGameTime,&LastTauntPrintedAt,"Computer Team");

	if (missionSave) {
		int i;

		// init bools
		if((b_array) && (b_count))
			for (i = 0; i < b_count; i++)
				b_array[i] = false;

		// init floats
		if((f_array) && (f_count))
			for (i = 0; i < f_count; i++)
				f_array[i] = 0.0f;

		// init handles
		if((h_array) && (h_count))
			for (i = 0; i < h_count; i++)
				h_array[i] = 0;

		// init ints
		if((i_array) && (i_count))
			for (i = 0; i < i_count; i++)
				i_array[i] = 0;

		Setup();
		return true;
	}

	bool ret = true;

	// bools
	if (b_array != NULL)
		ret=ret && Read(b_array, b_count);

	// floats
	if (f_array != NULL)
		ret=ret && Read(f_array, f_count);

	// Handles
	if (h_array != NULL)
		ret=ret && Read(h_array, h_count);

	// ints
	if (i_array != NULL)
		ret=ret && Read(i_array, i_count);

	return ret;
}

bool instantMission::Save(bool missionSave)
{
	// Always do this to hook up clients with the taunt engine as well.
	InitTaunts(&ElapsedGameTime,&LastTauntPrintedAt,"Computer Team");

	if (missionSave)
		return true;

	bool ret = true;

	// bools
	if (b_array != NULL)
		ret=ret && Write(b_array, b_count);

	// floats
	if (f_array != NULL)
		ret=ret && Write(f_array, f_count);

	// Handles
	if (h_array != NULL)
		ret=ret && Write(h_array, h_count);

	// ints
	if (i_array != NULL)
		ret=ret && Write(i_array, i_count);

	return ret;
}

bool instantMission::PostLoad(bool missionSave)
{
	if (missionSave)
		return true;

	if (h_array != NULL)
		ConvertHandles(h_array, h_count);

	return true;
}

int Rand(void)
{
	return (int)GetRandomFloat(32768);
}

void instantMission::DoGenericStrategy()
{
	time_count++;

	if (!start_done) {
		start_done=true;

		myGoal=GetInstantGoal();

		// Style notes: we copy out all the options.instant.* varbs we
		// want into class member variables the first chance we get. If
		// the user exits to windows, and reloads a savegame, only these
		// class member variables will be reloaded. The options.* will be
		// lost.

		// true if the user can respawn if died (kill/snipe)
		bCanRespawn = IFace_GetInteger("options.instant.bool0");

		// Clear this on entry. It might be filled in below.
		memset(mCustomAIPStr,0,sizeof(mCustomAIPStr));

		Aware_V13 = IFace_GetInteger("options.instant.awarev13");
		if(Aware_V13) {

			// Using custom AIPs? Read from string.
			IFace_GetString("options.instant.string0", mCustomAIPStr, mCustomAIPStrLen);

			CPUTeamRace = IFace_GetInteger("options.instant.hisrace");
			HumanTeamRace = IFace_GetInteger("options.instant.myrace");

			// Override whatever craft the BZN had for the player, make them
			// a scout of the correct race.
			Handle PlayerH = GetPlayerHandle();
			_ASSERTE(PlayerH != 0);
			int PlayerTeam = GetTeamNum(PlayerH);
			Vector PlayerPos;
			GetPosition(PlayerH,PlayerPos);

			char NewPlayerODF[32];
			sprintf(NewPlayerODF,"%cvscout",HumanTeamRace);

			char OldODFName[64];
			if(GetObjInfo(h, Get_CFG, OldODFName)) {
				// Got their cfg. Check if it's a pilot, and change to that if
				// the BZN wanted them to start as that.
				if(OldODFName[1] == 's')
					sprintf(NewPlayerODF,"%spilo",HumanTeamRace);
			}

			// Remove old object
			RemoveObject(PlayerH);
			// Build new object
			Handle NewPlayerH = BuildObject(NewPlayerODF, PlayerTeam, PlayerPos);

			SetAsUser(NewPlayerH,PlayerTeam);

		}
		else {
			// Old way of determining sides
			int mySide=GetInstantMySide();
			if(mySide == 1) { // CPU Scion, Human ISDF
				CPUTeamRace = kRACE_SCION;
				HumanTeamRace = kRACE_ISDF;
			}
			else {
				CPUTeamRace = kRACE_ISDF;
				HumanTeamRace = kRACE_SCION;
			}
		}

		myForce=GetInstantMyForce();
		compForce=GetInstantCompForce();
		difficulty=GetInstantDifficulty();



		if (myGoal==0)
		{
			strat_team=3;
			Ally(3,1);
			Ally(1,3);

			Handle atk;
			if (CPUTeamRace == kRACE_SCION) {
				atk=BuildObject("fvsent",comp_team,"tankEnemy1");
			}
			else
				atk=BuildObject("ivmisl",comp_team,"tankEnemy1");

			Attack(atk,player);  // spice up the start

			// add power ups here.. or have a function for it.  
		}
		else 
		{
			strat_team=1;
		}

		SetupExtraVehicles();

		// Build initial items for CPU team based on selections. The first
		// ODF listed is preferred, the second one really should be there
		// as a fallback

		// Note: contents of options.instant.string# not copied to member
		// varbs, as recommended above. Why not? It's only used in
		// creating an object, and then that object will save enough data
		// to reload successfully later.
		char CustomCPURecy[32];
		IFace_GetString("options.instant.string2", CustomCPURecy, sizeof(CustomCPURecy));
		if(CustomCPURecy[0]) 
			enemy_recycler=BuildStartingVehicle(comp_team,CPUTeamRace,CustomCPURecy,"*vrecy","RecyclerEnemy");
		else
			enemy_recycler=BuildStartingVehicle(comp_team,CPUTeamRace,"*vrecycpu","*vrecy","RecyclerEnemy");

		BuildStartingVehicle(comp_team,CPUTeamRace,"*vturr","*vturr","turretEnemy1");
		BuildStartingVehicle(comp_team,CPUTeamRace,"*vturr","*vturr","turretEnemy2");

		if (compForce>0) {
			BuildStartingVehicle(comp_team,CPUTeamRace,"*bspir","*vturr","gtow2");
			BuildStartingVehicle(comp_team,CPUTeamRace,"*bspir","*vturr","gtow3");
			BuildStartingVehicle(comp_team,CPUTeamRace,"*vsent","*vscout","SentryEnemy1");
			BuildStartingVehicle(comp_team,CPUTeamRace,"*vsent","*vscout","SentryEnemy2");
		}
		if (compForce>1) {
			BuildStartingVehicle(comp_team,CPUTeamRace,"*bspir","*vturr","gtow4");
			BuildStartingVehicle(comp_team,CPUTeamRace,"*bspir","*vturr","gtow5");
			BuildStartingVehicle(comp_team,CPUTeamRace,"*vtank","*vtank","tankEnemy1");
			BuildStartingVehicle(comp_team,CPUTeamRace,"*vtank","*vtank","tankEnemy2");
			BuildStartingVehicle(comp_team,CPUTeamRace,"*vtank","*vtank","tankEnemy3");
			BuildStartingVehicle(comp_team,CPUTeamRace,"*vtank","*vtank","TankEnemy3");
			BuildStartingVehicle(comp_team,CPUTeamRace,"*vsent","*vscout","SentryEnemy3");
		}
		BuildStartingVehicle(comp_team,CPUTeamRace,"*vscav","*vscav","ScavengerEnemy");


		// --------------------------------------------------------
		// Build initial items for Human team based on selections

		int grp=GetFirstEmptyGroup();

		// Note: contents of options.instant.string# not copied to member
		// varbs, as recommended above. Why not? It's only used in
		// creating an object, and then that object will save enough data
		// to reload successfully later.

		// If there is a custom human recycler, use that.
		char CustomHumanRecy[32];
		IFace_GetString("options.instant.string1", CustomHumanRecy, sizeof(CustomHumanRecy));
		if(CustomHumanRecy[0]) 
			recycler=BuildStartingVehicle(strat_team,HumanTeamRace,CustomHumanRecy,"*vrecy","Recycler",grp);
		else
			recycler=BuildStartingVehicle(strat_team,HumanTeamRace,"*vrecy","*vrecy","Recycler",grp);

		grp=GetFirstEmptyGroup();
		BuildStartingVehicle(strat_team,HumanTeamRace,"*vscav","*vscav","Scavenger",grp);

		grp=GetFirstEmptyGroup();
		BuildStartingVehicle(strat_team,HumanTeamRace,"*vturr","*vturr","Turret1",grp);
		BuildStartingVehicle(strat_team,HumanTeamRace,"*vturr","*vturr","Turret2",grp);
		BuildStartingVehicle(strat_team,HumanTeamRace,"*vturr","*vturr","Turret3",grp);

		grp=GetFirstEmptyGroup();
		BuildStartingVehicle(strat_team,HumanTeamRace,"*vscout","*vscout","Scout1",grp);
		BuildStartingVehicle(strat_team,HumanTeamRace,"*vscout","*vscout","Scout2",grp);
		BuildStartingVehicle(strat_team,HumanTeamRace,"*vscout","*vscout","Scout3",grp);

		grp=GetFirstEmptyGroup();
		BuildStartingVehicle(strat_team,HumanTeamRace,"*vtank","*vtank","Tank1",grp);
		BuildStartingVehicle(strat_team,HumanTeamRace,"*vtank","*vtank","Tank2",grp);
		BuildStartingVehicle(strat_team,HumanTeamRace,"*vtank","*vtank","Tank3",grp);
		if(myForce>1) {
			BuildStartingVehicle(strat_team,HumanTeamRace,"*vtank","*vtank","tank1",grp);
			BuildStartingVehicle(strat_team,HumanTeamRace,"*vtank","*vtank","tank2",grp);
			BuildStartingVehicle(strat_team,HumanTeamRace,"*vtank","*vtank","tank3",grp);
		}

		// Only old-style maps w/ opposite races can do thug mode
		if(!Aware_V13) {
			if (myGoal==0) {
				// check my team..
				if (HumanTeamRace == kRACE_ISDF) {
					SetPlan("isdfteam.aip",strat_team);
				}
				else {
					SetPlan("scionteam.aip",strat_team);
				}	
			}
		} // Not aware of 1.3

		SetCPUAIPlan(AIPType0);

		// do enemies
		//					ClearObjectives();
		//					AddObjective("isdf0501.otf",WHITE,5.0f);

		// power ups?
		BuildObject("apammo",0,"ammo1");
		BuildObject("apammo",0,"ammo2");
		BuildObject("apammo",0,"ammo3");
		BuildObject("aprepa",0,"repair1");
		BuildObject("aprepa",0,"repair2");
		BuildObject("aprepa",0,"repair3");
		// objectives .. later make this optional	
		//		CreateObjectives();
		SetScrap(comp_team,40);
		SetScrap(1,40);
	}	

	if (time_count%(30-(compForce*10))==0)  // this speeds up resources
		AddScrap(comp_team,1);
	if (time_count%(100-(myForce*20))==0)
		AddScrap(strat_team,1);
	if (time_count%friend_reinforcement_time==0)
	{
		/*
		BuildObject("ivtank",strat_team,"Tank1");
		BuildObject("ivtank",strat_team,"Tank2");
		BuildObject("ivtank",strat_team,"Tank3");
		BuildObject("ivturr",strat_team,"Turret1");
		BuildObject("ivturr",strat_team,"Turret2");
		BuildObject("ivturr",strat_team,"Turret3");
		BuildObject("ivscout",strat_team,"Scout1");
		BuildObject("ivscout",strat_team,"Scout2");
		BuildObject("ivscout",strat_team,"Scout3");
		*/
		// power ups?
		BuildObject("apammo",0,"ammo1");
		BuildObject("apammo",0,"ammo2");
		BuildObject("apammo",0,"ammo3");
		BuildObject("aprepa",0,"repair1");
		BuildObject("aprepa",0,"repair2");
		BuildObject("aprepa",0,"repair3");

	}

	if (myGoal==0) {
		powerup_counter++;
		if (powerup_counter%200==0) {
			if (!IsAround(powerup1)) {
				powerup1=BuildObject("apshdw",0,"power1");
			}
			if (!IsAround(powerup2)) {
				powerup2=BuildObject("apsstb",0,"power2");
			}
			if (!IsAround(powerup3)) {
				powerup3=BuildObject("apffld",0,"power3");
			}
			if (!IsAround(powerup4)) {
				powerup4=BuildObject("apspln",0,"power4");
			}
		}
	}

	// AIP switch logic
	if (!siege_on) {
		Handle enemy=GetNearestEnemy(enemy_recycler);

		char ODFName[64];
		if(GetObjInfo(enemy, Get_CFG, ODFName)) {
			// unit exists. Check skipping race name, and any suffix

			char ObjClass[64];
			GetObjInfo(enemy, Get_GOClass, ObjClass);
		
			// We want to ignore scavs that happen to drive near the CPU
			// recycler.  We're not in a siege if that's the case.
			if((stricmp(ObjClass, "CLASS_SCAVENGER") == 0) || (stricmp(ObjClass, "CLASS_SCAVENGERH") == 0)) {
				enemy = 0;
			}
		}

		if ((enemy) && (GetDistance(enemy,enemy_recycler) < kfSIEGE_DISTANCE)) { // under siege
			siege_counter++;
		} else {
			siege_counter=0;
		}

		if (siege_counter>450) { // 45 sec, if we don't change the sim rate :-)
			siege_on=true;
			SetCPUAIPlan(AIPTypeS);
		}
	}  // !siege_on
	else {
		Handle enemy=GetNearestEnemy(enemy_recycler);
		if (GetDistance(enemy,enemy_recycler) > kfSIEGE_DISTANCE) {
			SetCPUAIPlan(AIPTypeL);
			siege_on=false;
		}
	}  // siege_on
	// assault_test
	if ((!late_game) && (!siege_on) && (!anti_assault) && (assault_counter>2)) {
		anti_assault=true;
		SetCPUAIPlan(AIPTypeA);
	} else {
		if ((anti_assault) && (assault_counter<3)) {
			anti_assault=false;
			late_game=true;
			SetCPUAIPlan(AIPTypeL);
		}
	}

	TestObjectives();
}

EjectKillRetCodes instantMission::PlayerEjected(Handle DeadObjectHandle)
{
	// Always allow user to eject
	return DoEjectPilot;
}

// Function never called - NM 10/30/04
// EjectKillRetCodes instantMission::PlayerKilled(int KillersHandle)
// {
// 	return DoEjectPilot;  // Game over, man.
// }

// Local user died. Do (optional) respawn or fail mission
EjectKillRetCodes instantMission::PlayerDied(int DeadObjectHandle, bool bSniped)
{

	// Player (local or remote human) is dead
	if(!IsPerson(DeadObjectHandle)) {
		// Craft died. If it wasn't a snipe, then just kick the pilot out
		if(!bSniped)
			return DoEjectPilot;
	}

	// Local player got killed (pilot or snipe). Do respawn as needed

	if(bCanRespawn && IsAlive(recycler)) {
		// Respawn them near their recycler, up in the air.
		Vector Where;
		GetPosition(recycler, Where);

		Where=GetPositionNear(Where,10.0f,50.0f);
		Where.y += 50.0f;

		char PlayerODF[32];
		sprintf(PlayerODF,"%cspilo", HumanTeamRace);
		Handle PlayerH=BuildObject(PlayerODF,1,Where);
		SetAsUser(PlayerH,1);
		AddPilotByHandle(PlayerH);

		DoTaunt(TAUNTS_HumanShipDestroyed);
	}
	else {
		// User can't respawn. Game over, man
		FailMission(GetTime() + 3.0f);
	}

	// Both cases of the above report that we handled things
	return DLLHandled;
}

EjectKillRetCodes instantMission::ObjectKilled(int DeadObjectHandle, int KillersHandle)
{
	if(!IsPlayer(DeadObjectHandle)) { // J.Random AI-controlled Object is toast
		// Should we eject a pilot instead?
		bool bWasDeadPilot = IsPerson(DeadObjectHandle);

		if(!bWasDeadPilot)
			return DoEjectPilot;
		else
			return DLLHandled;
	}
	else {
		return PlayerDied(DeadObjectHandle, false);
	}
}

EjectKillRetCodes instantMission::ObjectSniped(int DeadObjectHandle, int KillersHandle)
{
	if(!IsPlayer(DeadObjectHandle)) {
		// J.Random AI-controlled Object is toast
		return DLLHandled;
	}
	else {
		return PlayerDied(DeadObjectHandle, true);
	} // Player dead
}

DLLBase * BuildMission(void)
{
	return new instantMission();
}

