#ifndef INSTANTMISSION_H__
#define INSTANTMISSION_H__

#include "..\Shared\DLLBase.h"

class instantMission : public DLLBase
{
private:
	enum AIPType {
		AIPType0=0,
		AIPType1,
		AIPType2,
		AIPType3,
		AIPTypeA,
		AIPTypeL,
		AIPTypeS,
		MAX_AIP_TYPE,
	};

	enum {
		// Number of CPU Scavs that'll trigger a cleanup
		MaxCPUScavs=15,
		
		// Number of scavs (vehicles) that'll be left around after a cleanup.
		MinCPUScavs=5,

		MAX_GOODIES=40,
	};

	// bools
	bool
		b_first,
		start_done,
		inspect_bunker,
		second_objective,
		picking_up[MAX_GOODIES],
		captured[MAX_GOODIES],
		siege_on,
		anti_assault,
		late_game,
		have_armory,
		SetFirstAIP,
		bPastAIP0, // true when they've built a recyler building, or we get bored
		bCanRespawn, // true if the user can respawn if died (kill/snipe)
		bCPUBuiltCommBunker,
		b_last;

	// floats
	float
		f_first,
		f,
		f_last;

	// handles
	Handle
		h_first,
		h,
		obj1,
		bunker,
		bunker2,
		player,
		foe1,
		foe2,
		foe3,
		walker,
		tug,
		recycler, // Human's recycler
		enemy_recycler, // CPU's recycler
		powerup1,
		powerup2,
		powerup3,
		powerup4,
		CPUScavList[MaxCPUScavs], // List of CPU scav handles, used during cleanup.
		h_last;

	// integers
	int
		i_first,
		strat_team,
		dll_flag,
		i,
		siege_counter,
		assault_counter,
		next_wave,
		next_spawn,
		time_count,
		post_bunker,
		enemy_reinforcement_time,
		friend_reinforcement_time,
		end_counter,
		comp_team,
		powerup_counter,
		turn_counter,
		myForce,
		myGoal,
		//		mySide, // replaced by CPUTeamRace,HumanTeamRace
		Aware_V13,
		CPUTeamRace,
		HumanTeamRace,

		compForce,
		difficulty,
		goalval,
		NumCPUScavs,

		ElapsedGameTime,
		LastTauntPrintedAt,

		// UGLY CODE ALERT -- storing a string in an array of ints. Done
		// because that simplifies the load/save code quite a lot.
		// Avert your eyes if you're with the coding style police.
		mCustomAIPInts[8], // == 32 bytes 

		i_last;

protected:
	bool *b_array;
	int b_count;

	float *f_array;
	int f_count;

	int *h_array;
	int h_count;

	int *i_array;
	int i_count;

private:
	char *mCustomAIPStr; // Points to mCustomAIPInts . Use this instead
	int mCustomAIPStrLen; // size, in bytes, of this buffer

public:
	// Calls made from the outside

	// Constructor
	instantMission(void);

	// Housekeeping functions
	bool Load(bool missionSave);
	bool PostLoad(bool missionSave);
	bool Save(bool missionSave);

	// Callback when an object has been built into the world.
	void AddObject(Handle h);
	void DeleteObject(Handle h);

	EjectKillRetCodes PlayerEjected(Handle DeadObjectHandle);
	EjectKillRetCodes ObjectKilled(int DeadObjectHandle, int KillersHandle);
	EjectKillRetCodes ObjectSniped(int DeadObjectHandle, int KillersHandle);
	void Execute(void);
	virtual void DoGenericStrategy();

private:
	// Rest of calls are internal functions, thus made private.

	virtual void Setup(void);
	virtual void CreateObjectives();
	virtual void TestObjectives();

	// Sets the current AIP based on passed-in AIP type, which sides these are
	void SetCPUAIPlan(AIPType Type);

	// And set up extra race-specific vehicles at pathpoints in the map.
	void SetupExtraVehicles(void);

	// Builds a starting vehicle on the specified team with either ODF1
	// (preferred, if found) or ODF2 as its name, modified by the
	// race. Sets group, if specified. Returns the handle
	Handle BuildStartingVehicle(int Team, int Race, char *ODF1, char *ODF2, char *Where, int aGrp = -1);

	// Local user died. Do (optional) respawn or fail mission
	EjectKillRetCodes PlayerDied(int DeadObjectHandle, bool bSniped);

}; // end of 'class instantMission'

#endif // INSTANTMISSION_H__
